%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                 		%                     
%Interpolación Cúbica de Restringida    %
%                                 		%                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parámetros de Entrada:

%x,y:Vectores que guardan los números de los pares ordenado (xi,yi)
%    para i=0,1,2,....,n donde x0<x1<x2<...<xn.

%newx:Vector de elementos para los cuales se estimarán mediante la Interpolación
%     Cúbica Restringida.

function[newy]=ICCons(x,y,newx)
	n1=length(x);%Obtenemos el número de elementos de x.
	n2=length(newx);%Obtenemos el número de elementos de newx.
	newy=zeros(n2,1);%Vector que guardará los nuevos valores de y.
	%Vectores necesarios para calcular los coeficientes de los polinomios. 
	h=zeros(n1-1,1);
	s=zeros(n1+3,1);
	dy=zeros(n1,1);
    a=zeros(n1-1,1);
    b=zeros(n1-1,1);
    c=zeros(n1-1,1);
    d=zeros(n1-1,1);
    %Calcuamos los coeficientes de los polinomio.
    for i=(1:n1-1)
    	s(i)=(y(i+1)-y(i))/(x(i+1)-x(i)) ;
   		h(i)=x(i+1)-x(i);
    end
	for i=(2:n1-1)
    	if(sign(s(i-1))~=sign(s(i)))
        	dy(i)=0;
    	else
        	dy(i)=2/((1/s(i))+(1/s(i-1)));
    	end
	end
	dy(1)=((3/2)*s(1))-((1/2)*dy(2));
	dy(n1)=((3/2)*s(n1-1))-((1/2)*dy(n1-1));
	for i=(1:n1-1)
    	a(i)=y(i);
    	b(i)=dy(i);
    	c(i)=(3*s(i)-dy(i+1)-2*dy(i))/(h(i));
    	d(i)=-(2*s(i)-dy(i+1)-dy(i))/(h(i)^2);
	end
	%Encontramos el nuevo punto para determinar a que polinomio pertenece.
	for i=(1:n2)
		distancia=abs(min(x)-max(x));
		for j=(1:n1)
			if(abs(newx(i)-x(j))<distancia)
				distancia=abs(newx(i)-x(j));
				indx=j;
			end
		end
		%Aproximamos los nuevos puntos y.
		if(newx(i)<x(indx))
			indx=indx-1;
			newy(i)=a(indx)+b(indx)*(newx(i)-x(indx))+c(indx)*(newx(i)-x(indx))^2+d(indx)*(newx(i)-x(indx))^3;
		elseif(newx(i)>x(indx))
			newy(i)=a(indx)+b(indx)*(newx(i)-x(indx))+c(indx)*(newx(i)-x(indx))^2+d(indx)*(newx(i)-x(indx))^3;
        elseif((newx(i)-x(indx))<10^-8)
            newy(i)=y(indx);
		end
	end
end

%Parámetros de Salida:

%newy:Nuevos puntos estimados mediante Interpolación Restringida.