%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                          %                     
%Interpolación de Hermite  %
%                          %                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parámetros de Entrada:

%x,y:Vectores que guardan los números de los pares ordenado (xi,yi)
%    para i=0,1,2,....,n donde x0<x1<x2<...<xn.


%dy:Vector que guarda las derivadas en los puntos(xi,yi).

%newx:Vector de elementos para los cuales se estimarán mediante la Interpolación
%     de Hermite.

function[newy]=IHermite(x,y,dy,newx)
	n1=length(x);%Obtenemos el número de elemntos de x.
	n2=length(newx);%Obtenemos el número de elemntos de newx.
	PHermite=zeros(2*n1-1,n2);%Matriz que guardará los valores de polinomio de Hermite.
    producto=1;%Variable para realizar el producto del polinomio.
	%Obtenemos los coeficientes de Polinomio de Hermite.
	Q=DDivididasH(x,y,dy);
	%Construimos el polinomio de Hermite para cada newx.
    for i=(1:n2)
        for k=(1:n1-1)
        	producto=producto*(newx(i)-x(k));
        	PHermite(2*k-1,i)=producto;
        	producto=producto*(newx(i)-x(k));
        	PHermite(2*k,i)=producto;
        end
        producto=producto*(newx(i)-x(n1));
        PHermite(2*n1-1,i)=producto;
        producto=1;
    end
    %Reajustamos la matriz PHermite.
    PHermite=[ones(1,n2);PHermite];
    newy=(Q'*PHermite)';%Obtenemos los nuevos valores de y.

end

%Parámetros de Salida:

%newy:Nuevos puntos estimados mediante el polinomio de Lagrange.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                  %                     
%Diferencias Divididas de Hermite  %
%                                  %                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parámetros de Entrada:

%x,y:Vectores que guardan los números de los pares ordenado (xi,yi)
%    para i=0,1,2,....,n donde x0<x1<x2<...<xn.

%dy:Vector que guarda las derivadas en los puntos(xi,yi).

function[Q]=DDivididasH(x,y,dy)
	n=length(x);%Obtenemos el número de elemntos de x.
	z=zeros(2*n,1);%Vector que guarda los nuevos valores zi para.
    Q=zeros(2*n,2*n);%Matriz que guarda las dierencias divididas de Newton.
    %Obtenemos la diferencias divididas de Hermite.
    for i=(1:n)
		z(2*i-1)=x(i);
		z(2*i)=x(i);
		Q(2*i-1,1)=y(i);
		Q(2*i,1)=y(i);
		Q(2*i,2)=dy(i);
        if(i>1)
            Q(2*i-1,2)=(Q(2*i-1,1)-Q(2*i-2,1))/(z(2*i-1)-z(2*i-2));
        end
    end
    for i=(3:2*n)
    	for j=(3:i)
    		Q(i,j)=(Q(i,j-1)-Q(i-1,j-1))/(z(i)-z(i-(j-1)));
    	end
    end
    %Obtenemos la diagonal de Q que son los valoores de los coeficientes 
    %del interpolante de Hermite.
    Q=diag(Q);
end

%Parámetros de Salida:

%ak:Vector que contiene los coeficientes del Interpolante de Hermite. 

