%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                          %                     
%Interpolación de Newton   %
%                          %                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parámetros de Entrada:

%x,y:Vectores que guardan los números de los pares ordenado (xi,yi)
%    para i=0,1,2,....,n donde x0<x1<x2<...<xn.

%newx:Vector de elementos para los cuales se estimarán mediante la Interpolación
%     Newton.

function[newy]=INewton(x,y,newx)
	n1=length(x);%Obtenemos el número de elemntos de x.
	n2=length(newx);%Obtenemos el número de elemntos de newx.
    PNewton=zeros(n1-1,n2);%Matriz que guardará los valores de polinomio de Newton.
    producto=1;%Variable para realizar el producto del polinomio.
    %Construimos el polinomio de Newton para cada newx.
    for i=(1:n2)
        for k=(1:n1-1)
            producto=producto*(newx(i)-x(k));%Obtenemos el producto.
            PNewton(k,i)=producto;%Guadamos los valores del polinomio. 
        end
        producto=1;
    end
    %Obtenemos los ak (Diferencias divididas).
     ak=DDivididasN(x,y);
    %Reajustamos la matriz PNewton.
    PNewton=[ones(1,n2);PNewton];
    newy=(ak'*PNewton)';%Obtenemos los nuevos valores de y.
end

%Parámetros de Salida:

%newy:Nuevos puntos estimados mediante el polinomio de Lagrange.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                  %                     
%Diferencias Divididas de Newton   %
%                                  %                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parámetros de Entrada:

%x,y:Vectores que guardan los números de los pares ordenado (xi,yi)
%    para i=0,1,2,....,n donde x0<x1<x2<...<xn.

function[ak]=DDivididasN(x,y)
	n=length(x);%Obtenemos el número de elemntos de x.
	ak=zeros(n,n);%Matriz que guarda las dierencias divididas de Newton.
	%Obtenemos la diferencias divididas de Newton.
    ak(:,1)=y;
    for i=(2:n)
		for j=(2:i)
			ak(i,j)=(ak(i,j-1)-ak(i-1,j-1))/(x(i)-x(i-(j-1)));
		end
    end
    %Obtenemos la diagonal de ak que son los valoores de los coeficientes.
    %del interpolante de Newton.
	ak=diag(ak);
end

%Parámetros de Salida:

%ak:Vector que contiene los coeficientes del Interpolante de Newton. 


