%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                 %                     
%Interpolación Cúbica Monótona    %
%                                 %                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parámetros de Entrada:

%x,y:Vectores que guardan los números de los pares ordenado (xi,yi)
%    para i=0,1,2,....,n donde x0<x1<x2<...<xn.

%newx:Vector de elementos para los cuales se estimarán mediante la Interpolación
%     Cúbica Monótona.

function[newy]=MPIH(x,y,newx)
	n1=length(x);%Obtenemos el número de elementos de x.
	n2=length(newx);%Obtenemos el número de elementos de newx.
	newy=zeros(n2,1);%Vector que guardará los nuevos valores de y.
	%Vectores necesarios para calcular los coeficientes de los polinomios. 
	h=zeros(n1-1,1);
	s=zeros(n1+3,1);
	dy=zeros(n1,1);
    a=zeros(n1-1,1);
    b=zeros(n1-1,1);
    c=zeros(n1-1,1);
    d=zeros(n1-1,1);

    %Calculamos los coeficientes del polinomio.
    for i=(1:n1-1)
    	s(i)=(y(i+1)-y(i))/(x(i+1)-x(i));
   		h(i)=x(i+1)-x(i);
    end
    for i=(2:n1-1)
    	if ((s(i-1)*s(i))<=0)
        	dy(i)=0;
    	else
        	w1=((h(i-1)+2*h(i))/(3*(h(i)+h(i-1))))*(1/s(i-1));
        	w2=((2*h(i-1)+h(i))/(3*(h(i)+h(i-1))))*(1/s(i));
        	dy(i)=1/(w1+w2);
    	end
  	end
  	g1=(((2*h(1)+h(2))/(h(1)+h(2)))*s(1))-((h(1)/(h(1)+h(2)))*s(2));
	gn1=(((2*h(n1-1)+h(n1-2))/(h(n1-1)+h(n1-2)))*s(n1-1))-((h(n1-1)/(h(n1-1)+h(n1-2)))*s(n1-2));
	if(g1*s(1)<=0)
    	dy(1)=0;
	elseif(s(1)*s(2)<=0 && abs(g1)>3*abs(s(1)))
    	dy(1)=3*s(1);
	else
    	dy(1)=g1;
	end
	if(gn1*s(n1-1)<=0)
    	dy(n1)=0;
	elseif(s(n1-1)*s(n1-2)<=0 && abs(gn1)>3*abs(s(n1-1)))
    	dy(n1)=3*s(n1-1);
	else
    	dy(n1)=gn1;
	end
	for i=(1:n1-1)
    	a(i)=y(i);
    	b(i)=dy(i);
    	c(i)=(3*s(i)-dy(i+1)-2*dy(i))/(h(i));
    	d(i)=-(2*s(i)-dy(i+1)-dy(i))/(h(i)^2);
	end
	%Encontramos el nuevo punto para determinar a que polinomio pertenece.
	for i=(1:n2)
		distancia=abs(min(x)-max(x));
		for j=(1:n1)
			if(abs(newx(i)-x(j))<distancia)
				distancia=abs(newx(i)-x(j));
				indx=j;
			end
		end
		%Aproximamos los nuevos puntos y.
		if(newx(i)<x(indx))
			indx=indx-1;
			newy(i)=a(indx)+b(indx)*(newx(i)-x(indx))+c(indx)*(newx(i)-x(indx))^2+d(indx)*(newx(i)-x(indx))^3;
		elseif(newx(i)>x(indx))
			newy(i)=a(indx)+b(indx)*(newx(i)-x(indx))+c(indx)*(newx(i)-x(indx))^2+d(indx)*(newx(i)-x(indx))^3;
        elseif((newx(i)-x(indx))<10^-8)
            newy(i)=y(indx);
		end
	end
end
%Parámetros de Salida:

%newy:Nuevos puntos estimados mediante Interpolación Cúbica Monótona.