%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                    %                     
%Interpolación Spline Cúbico Completo%
%                                    %                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parámetros de Entrada:

%x,y:Vectores que guardan los números de los pares ordenado (xi,yi)
%    para i=0,1,2,....,n donde x0<x1<x2<...<xn.

%newx:Vector de elementos para los cuales se estimarán mediante la Interpolación
%     Spline Cúbico Completo.

function[newy]=SCCompleto(x,y,dfa,dfb,newx)
	n1=length(x);%Obtenemos el número de elementos de x.
	n2=length(newx);%Obtenemos el número de elementos de newx.
    newy=zeros(n2,1);%Vector que guardará los nuevos valores de y.

    %Calcuamos z para encontrar los coeficientes del polinomio.
    h=zeros(n1-1,1);
	a=zeros(n1,1);
	l=zeros(n1,1);
	u=zeros(n1,1);
	zp=zeros(n1,1);
	z=zeros(n1,1);
	for i=(1:n1-1)
		h(i)=x(i+1)-x(i);
	end
	a(1)=(3*(y(2)-y(1)))/h(1)-3*dfa;
	a(n1)=3*dfb-3*((a(n1)-a(n1-1))/h(n1-1));
	for i=(2:n1-1)
		a(i)=(3/h(i))*(y(i+1)-y(i))-(3/h(i-1))*(y(i)-y(i-1));
	end
	l(1)=2*h(1);
	u(1)=0.5;
	zp(1)=a(1)/l(1);
	for i=(2:n1-1)
		l(i)=2*(x(i+1)-x(i-1))-h(i-1)*u(i-1);
		u(i)=h(i)/l(i);
		zp(i)=(a(i)-h(i-1)*zp(i-1))/l(i);
	end
	l(n1)=h(n1-1)*(2-u(n1-1));
	zp(n1)=(a(n1)-h(n1-1)*zp(n1-1))/l(n1);
	z(n1)=zp(n1);
	for i=(n1-1:-1:1)
		z(i)=zp(i)-u(i)*z(i+1);
	end
	%Encontramos el nuevo punto para determinar a que polinomio pertenece.
	for i=(1:n2)
		distancia=abs(min(x)-max(x));
		for j=(1:n1)
			if(abs(newx(i)-x(j))<distancia)
				distancia=abs(newx(i)-x(j));
				indx=j;
			end
		end
		%Aproximamos los nuevos puntos y.
		if(newx(i)<x(indx))
			indx=indx-1;
			A=(1/(6*h(indx)))*(z(indx+1)-z(indx));
			B=z(indx)/2;
			C=(-h(indx)/6)*(z(indx+1))-(h(indx)/3)*z(indx)+(1/h(indx))*(y(indx+1)-y(indx));
			newy(i)=y(indx)+(newx(i)-x(indx))*(C+(newx(i)-x(indx))*(B+(newx(i)-x(indx))*A));
		elseif(newx(i)>x(indx))
			A=(1/(6*h(indx)))*(z(indx+1)-z(indx));
			B=z(indx)/2;
			C=(-h(indx)/6)*(z(indx+1))-(h(indx)/3)*z(indx)+(1/h(indx))*(y(indx+1)-y(indx));
			newy(i)=y(indx)+(newx(i)-x(indx))*(C+(newx(i)-x(indx))*(B+(newx(i)-x(indx))*A));
        elseif((newx(i)-x(indx))<10^-8)
            newy(i)=y(indx);
		end
	end

end

%Parámetros de Salida:

%newy:Nuevos puntos estimados mediante Spline Completo.