%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                       %                  
%Interpolación Spline Cúbico Knot-A-Not %
%                                       %                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parámetros de Entrada:

%x,y:Vectores que guardan los números de los pares ordenado (xi,yi)
%    para i=0,1,2,....,n donde x0<x1<x2<...<xn.

%newx:Vector de elementos para los cuales se estimarán mediante la Interpolación
%     Spline Cúbico Knot-A-Not.

function[newy]=SCNKnot(x,y,newx)
	n1=length(x);%Obtenemos el número de elementos de x.
	n2=length(newx);%Obtenemos el número de elementos de newx.
	newy=zeros(n2,1);%Vector que guardará los nuevos valores de y.

	%Calcuamos los coeficientes de los polinomio.
	z=zeros(n1,1);
	h=zeros(n1-1,1);
	b=zeros(n1,1);
    for i=(1:n1-1)
		h(i)=x(i+1)-x(i);
        if(i>2)
			b(i)=((6/h(i))*(y(i+1)-y(i)))-((6/h(i-1))*(y(i)-y(i-1)));
        end
    end
    e=zeros(n1-2,1);
	a=zeros(n1-1,1);
	d=zeros(n1,1);
	c=zeros(n1-1,1);
	f=zeros(n1-2,1);
	e(n1-2)=-h(n1-1);
	a(n1-1)=h(n1-2)+h(n1-1);
	d(n1)=-h(n1-2);
	d(1)=-h(2);
	c(1)=h(1)+h(2);
	f(1)=-h(1);
	for i=(2:n1-1)
		a(i-1)=h(i-1);
		d(i)=2*(h(i)+h(i-1));
		c(i)=h(i);
	end
	b(1)=0;
	b(n1)=0;

	r=a(1);
	s=a(2);
	t=e(1);
	for i=(2:n1-1)
		zmult=r/d(i-1);
		d(i)=d(i)-zmult*c(i-1);
		c(i)=c(i)-zmult*f(i-1);
		b(i)=b(i)-zmult*b(i-1);
		zmult=t/d(i-1);
		r=s-zmult*c(i-1);
		d(i+1)=d(i+1)-zmult*f(i-1);
		b(i+1)=b(i+1)-zmult*b(i-1);
        if(i<n1-2)            
            s=a(i+1);
            t=e(i);
        end
	end
	zmult=r/d(n1-1);
	d(n1)=d(n1)-zmult*c(n1-1);
	z(n1)=(b(n1)-zmult*b(n1-1))/d(n1);
	z(n1-1)=(b(n1-1)-c(n1-1)*z(n1))/d(n1-1);
	for i=(n1-2:-1:1)
		z(i)=(b(i)-f(i)*z(i+2)-c(i)*z(i+1))/d(i);
	end
	%Encontramos el nuevo punto para determinar a que polinomio pertenece.
	for i=(1:n2)
		distancia=abs(min(x)-max(x));
		for j=(1:n1)
			if(abs(newx(i)-x(j))<distancia)
				distancia=abs(newx(i)-x(j));
				indx=j;
			end
		end
		%Aproximamos los nuevos puntos y.
		if(newx(i)<x(indx))
			indx=indx-1;
			A=(1/(6*h(indx)))*(z(indx+1)-z(indx));
			B=z(indx)/2;
			C=(-h(indx)/6)*(z(indx+1))-(h(indx)/3)*z(indx)+(1/h(indx))*(y(indx+1)-y(indx));
			newy(i)=y(indx)+(newx(i)-x(indx))*(C+(newx(i)-x(indx))*(B+(newx(i)-x(indx))*A));
		elseif(newx(i)>x(indx))
			A=(1/(6*h(indx)))*(z(indx+1)-z(indx));
			B=z(indx)/2;
			C=(-h(indx)/6)*(z(indx+1))-(h(indx)/3)*z(indx)+(1/h(indx))*(y(indx+1)-y(indx));
			newy(i)=y(indx)+(newx(i)-x(indx))*(C+(newx(i)-x(indx))*(B+(newx(i)-x(indx))*A));
        elseif((newx(i)-x(indx))<10^-8)
            newy(i)=y(indx);
		end
	end
end

%Parámetros de Salida:

%newy:Nuevos puntos estimados mediante Spline Cúbico Knot-A-Not.
			