%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                    %                     
%Interpolación Spline Cúbico Natural %
%                                    %                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parámetros de Entrada:

%x,y:Vectores que guardan los números de los pares ordenado (xi,yi)
%    para i=0,1,2,....,n donde x0<x1<x2<...<xn.

%newx:Vector de elementos para los cuales se estimarán mediante la Interpolación
%     Spline Cúbico Natural.

function[newy]=SCNatural(x,y,newx)
	n1=length(x);%Obtenemos el número de elementos de x.
	n2=length(newx);%Obtenemos el número de elementos de newx.
	newy=zeros(n2,1);%Vector que guardará los nuevos valores de y.

	%Calcuamos los coeficientes de los polinomio.
	z=zeros(n1,1);
	h=zeros(n1-1,1);
	b=zeros(n1-1,1);
	u=zeros(n1-2,1);
	v=zeros(n1-2,1);
	for i=(1:n1-1)
		h(i)=x(i+1)-x(i);
		b(i)=6*(y(i+1)-y(i))/h(i);
	end
	u(1)=2*(h(1)+h(2));
	v(1)=b(2)-b(1);
	for i=(2:n1-2)
		u(i)=2*(h(i)+h(i+1))-(h(i)^2)/u(i-1);
		v(i)=b(i+1)-b(i)-(h(i)*v(i-1))/u(i-1);
	end
	z(n1)=0;
	for i=(n1-1:-1:2)
		z(i)=(v(i-1)-(h(i)*z(i+1)))/u(i-1);
	end
	z(1)=0;
	%Encontramos el nuevo punto para determinar a que polinomio pertenece.
	for i=(1:n2)
		distancia=abs(min(x)-max(x));
		for j=(1:n1)
			if(abs(newx(i)-x(j))<distancia)
				distancia=abs(newx(i)-x(j));
				indx=j;
			end
		end
		%Aproximamos los nuevos puntos y.
		if(newx(i)<x(indx))
			indx=indx-1;
			A=(1/(6*h(indx)))*(z(indx+1)-z(indx));
			B=z(indx)/2;
			C=(-h(indx)/6)*(z(indx+1))-(h(indx)/3)*z(indx)+(1/h(indx))*(y(indx+1)-y(indx));
			newy(i)=y(indx)+(newx(i)-x(indx))*(C+(newx(i)-x(indx))*(B+(newx(i)-x(indx))*A));
		elseif(newx(i)>x(indx))
			A=(1/(6*h(indx)))*(z(indx+1)-z(indx));
			B=z(indx)/2;
			C=(-h(indx)/6)*(z(indx+1))-(h(indx)/3)*z(indx)+(1/h(indx))*(y(indx+1)-y(indx));
			newy(i)=y(indx)+(newx(i)-x(indx))*(C+(newx(i)-x(indx))*(B+(newx(i)-x(indx))*A));
        elseif((newx(i)-x(indx))<10^-8)
            newy(i)=y(indx);
		end
	end
end

%Parámetros de Salida:

%newy:Nuevos puntos estimados mediante Spline Cúbico Natural.
			