%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                 	%                     
%Interpolación Bivariada de Akima  	%
%                                 	%                    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parámetros de entrada:

%ncp: Número de puntos cercanos para la estimación de las derivadas parciales.

%xd,yd,zd: Vectores que contiene los puntos (xi,yi,zi).

%xi,yi: Vectores que contienes los puntos para (xi,yi) para ser interpolados.

function [zi,iwk,wk] = idbvip(ncp,xd,yd,zd,xi,yi)
    
	%Variables globales.
	
    global idlc
    global idpi
    global nit
    global itpv
    
	%Variables para el cálculo.
	md=1;
    ndp=length(xd);
    nip=length(xi);
    zi=zeros(nip,1);
    iwk=zeros((max(31,27+ncp)*ndp+nip),1);
    
    %Proceso preeliminar.
	
    md0 = md;
    ncp0 = ncp;
    ndp0 = ndp;
    nip0 = nip;
	
	%Almacenamos en wk e iwk información necesaria para la interpolación.
    
    if ( md0 < 2 ) 

        iwk(1) = ncp0;
        iwk(2) = ndp0;
    
    end
    
    if ( md0 < 3 )

        iwk(3) = nip;

    end
    
    jwipt = 16;
    jwiwl = 6 * ndp0 + 1;
    jwiwk = jwiwl;
    jwipl = 24 * ndp0 + 1;
    jwiwp = 30 * ndp0 + 1;
    jwipc = 27 * ndp0 + 1;
    jwit0 = max ( 31, 27 + ncp0 ) * ndp0;
    
	%Obtenemos la malla triángular y su información.
	
    if( md0 == 1 )
        [nt, iwk11, nl, iwk12, iwk13, iwk14, wk]=idtang(xd,yd);
        iwk(jwipt:jwipt+length(iwk11)-1)=iwk11;
        iwk(jwipl:jwipl+length(iwk12)-1)=iwk12;
        iwk(jwiwl:jwiwl+length(iwk13)-1)=iwk13;
        iwk(jwiwp:jwiwp+length(iwk14)-1)=iwk14;
        iwk(5) = nt;
        iwk(6) = nl;
        wk=[wk(1:ndp);zeros(7*ndp,1)];
        
        if ( nt == 0 ) 
            return;
        end
        
    end
    
	%Obtenemos el índice de los puntos cercanos.
	
    if ( md0 <= 1 )

        [iwk21]=idcldp (xd, yd, ncp);
        iwk(jwipc:jwipc+length(iwk21)-1)=iwk21;
        if ( iwk(jwipc) == 0 ) 
            return;
        end 

    end 
    
	%Obtenemos la ubucación de cada (xi,yi) a ser interpolado en la malla triángular.

    if ( md0 ~= 3 )

        nit = 0;
        jwit = jwit0;
        for iip = (1:nip0)
            jwit = jwit + 1;
            [iwk(jwit),iwk41, wk]=idlctn( xd, yd, nt,iwk(jwipt:jwipt+3*nt-1),nl,iwk(jwipl:jwipl+3*nl-1), xi(iip), yi(iip),iwk(jwiwk:jwiwk+18*ndp-1),wk);
            iwk(jwiwk:jwiwk+18*ndp-1)=iwk41;
           
        end 

	%Obtenemos la aproximación a las derivadas parciales zx,zy,zxx,zxy,zyy.
	
    end 
    
    [wk1]=idpdrv(xd,yd,zd,ncp0,iwk21);
    wk=[wk1;wk(length(wk1)+1:8*ndp)];
    itpv = 0;
    jwit = jwit0;
    iwk51=iwk(jwipt:jwipt+3*nt-1);
    iwk52=iwk(jwipl:jwipl+3*nl-1);
	
	%Obtenemos la interpolación de Bivariada de Akima de los los puntos (xi,yi).
	
    for iip = (1:nip0)
        jwit = jwit + 1;
        [zi(iip) ]=idptip ( xd, yd, zd, nt, iwk51, nl, iwk52, wk1,iwk(jwit), xi(iip), yi(iip));
    end 
     
end

%Parámetros de Salida:

%zi: Vector que contiene el resultado de la Interpolación Bivariada de Akima.

%iwk,wk: Vectores que contiene información del proceso de interpolación.

