%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                             %             
%Función que aproxima las derivadas parciales %
%                                             %                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parámetros de Entrada:

%xd,yd,zd:Vectores que guardan los números de (xi,yi,zi).

%ncp: Número de puntos cercanos requeridos (mínimo 3).

%ipc: Índice de puntos cercanos.

function[pd]=idpdrv(xd,yd,zd,ncp,ipc)
	
	%Proceso preeliminar.
    
	ndp=length(xd);
    pd=zeros(5*ndp,1);
	ndp0=ndp;
  	ncp0=ncp;
  	ncpm1=ncp0-1;

  	%Estimamos las derivadas parciales ZX y ZY.

  	for ip0=(1:ndp0)
  		x0 = xd(ip0);
        y0 = yd(ip0);
        z0 = zd(ip0);
        nmx = 0;
        nmy = 0;
        nmz = 0;
        jipc0 = ncp0*(ip0-1);
        for ic1=(1:ncpm1)
        	jipc = jipc0+ic1;
          	ipi = ipc(jipc);
          	dx1 = xd(ipi)-x0;
          	dy1 = yd(ipi)-y0;
          	dz1 = zd(ipi)-z0;
          	ic2mn = ic1+1;
        	for ic2=(ic2mn:ncp0)
        		jipc = jipc0+ic2;
            	ipi = ipc(jipc);
            	dx2 = xd(ipi)-x0;
            	dy2 = yd(ipi)-y0;
            	dnmz = dx1*dy2-dy1*dx2;
            	if (dnmz== 0)
              		continue;
            	end
            	dz2 = zd(ipi)-z0;
            	dnmx = dy1*dz2-dz1*dy2;
            	dnmy = dz1*dx2-dx1*dz2;
            	if (dnmz < 0)
              		dnmx =-dnmx;
              		dnmy =-dnmy;
              		dnmz =-dnmz;
            	end
            	nmx = nmx+dnmx;
            	nmy = nmy+dnmy;
            	nmz = nmz+dnmz;
        	end
        end
        jpd0 = 5*ip0;
        pd(jpd0-4) = -nmx/nmz;
        pd(jpd0-3) = -nmy/nmz;
  	end

  	%Estimamos las derivadas parciales ZXX, ZXY, y ZYY.

  	for ip0 =(1:ndp0)
        x0 = xd(ip0);
        jpd0 = 5*ip0;
        y0 = yd(ip0);
        zx0 = pd(jpd0-4);
        zy0 = pd(jpd0-3);
        nmxx = 0;
        nmxy = 0;
        nmyx = 0;
        nmyy = 0;
        nmz  = 0;
        jipc0 = ncp0*(ip0-1);
        for ic1 =(1:ncpm1)
        	jipc = jipc0+ic1;
          	ipi = ipc(jipc);
          	dx1 = xd(ipi)-x0;
          	dy1 = yd(ipi)-y0;
          	jpd = 5*ipi;
          	dzx1 = pd(jpd-4)-zx0;
          	dzy1 = pd(jpd-3)-zy0;
          	ic2mn = ic1+1;
          	for ic2 =(ic2mn:ncp0)
          		jipc = jipc0+ic2;
            	ipi = ipc(jipc);
            	dx2 = xd(ipi)-x0;
            	dy2 = yd(ipi)-y0;
            	dnmz  = dx1*dy2 -dy1*dx2;
            	if ( dnmz == 0 )
              		continue;
            	end
            	jpd = 5 * ipi;
            	dzx2 = pd(jpd-4) - zx0;
            	dzy2 = pd(jpd-3) - zy0;
            	dnmxx = dy1 * dzx2 - dzx1 * dy2;
            	dnmxy = dzx1 * dx2 - dx1 * dzx2;
            	dnmyx = dy1 * dzy2 - dzy1 * dy2;
            	dnmyy = dzy1 * dx2 - dx1 * dzy2;
            	if ( dnmz < 0) 
              		dnmxx = -dnmxx;
              		dnmxy = -dnmxy;
              		dnmyx = -dnmyx;
              		dnmyy = -dnmyy;
              		dnmz  = -dnmz;
            	end 
            	nmxx = nmxx+dnmxx;
            	nmxy = nmxy+dnmxy;
            	nmyx = nmyx+dnmyx;
            	nmyy = nmyy+dnmyy;
            	nmz  = nmz +dnmz;
          	end
        end
        pd(jpd0-2) = - nmxx / nmz;
        pd(jpd0-1) = - ( nmxy + nmyx ) / ( 2 * nmz );
        pd(jpd0)   = - nmyy / nmz;
  	end
end

%Parámetros de Salida:

%pd: Aproximación a las derivadas parciales ZX,ZY,ZXX,ZXY,ZYY.
