%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                 	%                     
%Interpolación Bivariada de Akima   %
%                                 	%                    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parametros de entrada:

%xd,yd,zd: Vectores que contienes los los valores (xi,yi,zi) de cada punto. 

%nt:Número de triángulos.

%ipt:Índice de los vértices de los triángulos.

%nl: Número de lineas que conforman en borde de la triangular.

%ipl:Índice de los puntos que conforman las nl.

%pdd: Vector que contiene la aproximación las derivadas parciales zx,zy,zxx,zxy,zyy.

%iti:Índice del triángula donde se encuentra el punto que se va interpolar.

%xii,yii: Punto (xi,yi) que será interpoado. 


function [zii] = idptip(xd,yd,zd,nt,ipt,nl,ipl,pdd,iti,xii,yii)
	
	%Variables globales.
	
	global idpi
	global itpv
	
	global	 a	
	global	 aa	
	global	 ab	
	global	 ac	
	global	 act2
	global	 ad	
	global	 adbc
	global	 ap	
	global	 b	
	global	 bb	
	global	 bc	
	global	 bdt2
	global	 bp	
	global	 c	
	global	 cc	
	global	 cd	
	global	 cp	
	global	 csuv
	global	 d	
	global	 dd	
	global	 dlt
	global	 dp	
	global	 dx	
	global	 dy	
	global	 g1	
	global	 g2	
	global	 h1	
	global	 h2	
	global	 h3	
	global	 i	
	global	 idp
	global	 il1
	global	 il2
	global	 it0
	global	 jipl
	global	 jipt
	global	 jpd
	global	 jpdd
	global	 kpd
	global	 lu	
	global	 lv	
	global	 ntl
	global	 p0	
	global	 p00
	global	 p01
	global	 p02
	global	 p03
	global	 p04
	global	 p05
	global	 p1
	global	 p10
	global	 p11
	global	 p12
	global	 p13
	global	 p14
	global	 p2
	global	 p20
	global	 p21
	global	 p22
	global	 p23
	global	 p3
	global	 p30
	global	 p31
	global	 p32
	global	 p4
	global	 p40
	global	 p41
	global	 p5
	global	 p50
	global	 thsv
	global	 thus
	global	 thuv
	global	 thxu
	global	 u
	global	 v
	global	 x0
	global	 y0

	%Vectores utilizados para el proceso.
	
	pd=zeros(15,1);
	x=zeros(3,1);
	y=zeros(3,1);
	z=zeros(3,1);
	zii	=0;
	zu=zeros(3,1);
	zuu=zeros(3,1);
	zuv=zeros(3,1);
	zv=zeros(3,1);
	zvv=zeros(3,1);
    %Proceso preeliminar.
    it0 =floor( iti);
    ntl = nt+nl;

	%Comenzamos con la interpolación.
	
    if(it0<=ntl)
        if(it0~=itpv)
            jipt = 3*(it0-1);
            jpd = 0;
            for i = (1:3)
                jipt = jipt+1;
                idp = ipt(jipt);
                x(i) = xd(idp);
                y(i) = yd(idp);
                z(i) = zd(idp);
                jpdd = 5*(idp-1);
                for kpd = (1:5)
                    jpd = jpd+1;
                    jpdd = jpdd+1;
                    pd(jpd) = pdd(jpdd);
                end
            end
            
            %Realizamos la transformación UV.
            
            x0 = x(1);
            y0 = y(1);
            a = x(2)-x0;
            b = x(3)-x0;
            c = y(2)-y0;
            d = y(3)-y0;
            ad = a*d;
            bc = b*c;
            dlt = ad-bc;
            ap =  d/dlt;
            bp = -b/dlt;
            cp = -c/dlt;
            dp =  a/dlt;
 
            %Convertimos las derivadas parciales al sistema UV.
            
            aa = a*a;
            act2 = 2*a*c;
            cc = c*c;
            ab = a*b;
            adbc = ad+bc;
            cd = c*d;
            bb = b*b;
            bdt2 = 2*b*d;
            dd = d*d;
            
            for i = (1:3)
                jpd = 5*i;
                zu(i) = a*pd(jpd-4)+c*pd(jpd-3);
                zv(i) = b*pd(jpd-4)+d*pd(jpd-3);
                zuu(i) = aa*pd(jpd-2)+act2*pd(jpd-1)+cc*pd(jpd);
                zuv(i) = ab*pd(jpd-2)+adbc*pd(jpd-1)+cd*pd(jpd);
                zvv(i) = bb*pd(jpd-2)+bdt2*pd(jpd-1)+dd*pd(jpd);
            end 
            
            %Calculamos los coeficientes del polinomio.
            
            p00 = z(1);
            p10 = zu(1);
            p01 = zv(1);
            p20 = 0.5*zuu(1);
            p11 = zuv(1);
            p02 = 0.5*zvv(1);
            h1 = z(2)-p00-p10-p20;
            h2 = zu(2)-p10-zuu(1);
            h3 = zuu(2)-zuu(1);
            p30 =  10*h1-4*h2+0.5*h3;
            p40 = -15.0D+00*h1+7.0D+00*h2-h3;
            p50 =   6*h1-3*h2+0.5*h3;
            p5=p50;
            h1 = z(3)-p00-p01-p02;
            h2 = zv(3)-p01-zvv(1);
            h3 = zvv(3)-zvv(1);
            p03 =  10*h1-4*h2+0.5*h3;
            p04 = -15*h1+7*h2-h3;
            p05 =   6*h1-3*h2+0.5*h3;
            lu = sqrt(aa+cc);
            lv = sqrt(bb+dd);
            thxu = atan2(c,a);
            thuv = atan2(d,b)-thxu;
            csuv = cos(thuv);
            p41 = 5*lv*csuv/lu*p50;
            p14 = 5*lu*csuv/lv*p05;
            h1 = zv(2)-p01-p11-p41;
            h2 = zuv(2)-p11-4*p41;
            p21 =  3*h1-h2;
            p31 = -2*h1+h2;
            h1 = zu(3)-p10-p11-p14;
            h2 = zuv(3)-p11-4.0D+00*p14;
            p12 =  3*h1-h2;
            p13 = -2*h1+h2;
            thus = atan2(d-c,b-a)-thxu;
            thsv = thuv-thus;
            aa =  sin(thsv)/lu;
            bb = -cos(thsv)/lu;
            cc =  sin(thus)/lv;
            dd =  cos(thus)/lv;
            ac = aa*cc;
            ad = aa*dd;
            bc = bb*cc;
            g1 = aa*ac*(3*bc+2*ad);
            g2 = cc*ac*(3*ad+2*bc);
            h1 = -aa*aa*aa*(5*aa*bb*p50+(4*bc+ad)*p41)-cc*cc*cc*(5*cc*dd*p05+(4*ad+bc)*p14);
            h2 = 0.5*zvv(2)-p02-p12;
            h3 = 0.5*zuu(3)-p20-p21;
            p22 = (g1*h2+g2*h3-h1)/(g1+g2);
            p32 = h2-p22;
            p23 = h3-p22;
            itpv = it0;
            
            %Convertimos xii y yii al sistema UV.
            
            dx = xii-x0;
            dy = yii-y0;
            u = ap*dx+bp*dy;
            v = cp*dx+dp*dy;

            %Evaluamos el polinomio.
            
            p0 = p00+v*(p01+v*(p02+v*(p03+v*(p04+v*p05))));
            p1 = p10+v*(p11+v*(p12+v*(p13+v*p14)));
            p2 = p20+v*(p21+v*(p22+v*p23));
            p3 = p30+v*(p31+v*p32);
            p4 = p40+v*p41;
			
			%Encontramos el punto buscado.
            
			zii = p0+u*(p1+u*(p2+u*(p3+u*(p4+u*p5))));
			
            return;
            
        elseif(it0==itpv)
            
            %Convertimos xii y yii al sistema UV.
            
            dx = xii-x0;
            dy = yii-y0;
            u = ap*dx+bp*dy;
            v = cp*dx+dp*dy;
            
            %Evaluamos el polinomio.
            
            p0 = p00+v*(p01+v*(p02+v*(p03+v*(p04+v*p05))));
            p1 = p10+v*(p11+v*(p12+v*(p13+v*p14)));
            p2 = p20+v*(p21+v*(p22+v*p23));
            p3 = p30+v*(p31+v*p32);
            p4 = p40+v*p41;
			
			%Encontramos el punto buscado.
			
            zii = p0+u*(p1+u*(p2+u*(p3+u*(p4+u*p5))));                      
            return;
        end
        
    else
        il1 = floor(it0/ntl);
        il2 = floor(it0-il1*ntl);
        if(il1==il2) 
            %Calculamos el valor de zii.
            if(it0~=itpv)
                
                %Usamos las derivadas parciales.
                jipl = 3*(il1-1);
                jpd = 0;

                for i = (1:2)

                    jipl = jipl+1;
                    idp = ipl(jipl);
                    x(i) = xd(idp);
                    y(i) = yd(idp);
                    z(i) = zd(idp);
                    jpdd = 5*(idp-1);

                    for kpd = (1:5)
                        jpd = jpd+1;
                        jpdd = jpdd+1;
                        pd(jpd) = pdd(jpdd);
                    end 

                end
                
                %Creamos la conversión al sistema UV.
                
                x0 = x(1);
                y0 = y(1);
                a = y(2)-y(1);
                b = x(2)-x(1);
                c = -b;
                d = a;
                ad = a*d;
                bc = b*c;
                dlt = ad-bc;
                ap =  d/dlt;
                bp = -b/dlt;
                cp = -bp;
                dp =  ap;
                
                %Convertimos las derivadas parciales a sistema UV.
                
                aa = a*a;
                act2 = 2*a*c;
                cc = c*c;
                ab = a*b;
                adbc = ad+bc;
                cd = c*d;
                bb = b*b;
                bdt2 = 2*b*d;
                dd = d*d;
                
                for i = (1:2)
                    jpd = 5*i;
                    zu(i) = a*pd(jpd-4)+c*pd(jpd-3);
                    zv(i) = b*pd(jpd-4)+d*pd(jpd-3);
                    zuu(i) = aa*pd(jpd-2)+act2*pd(jpd-1)+cc*pd(jpd);
                    zuv(i) = ab*pd(jpd-2)+adbc*pd(jpd-1)+cd*pd(jpd);
                    zvv(i) = bb*pd(jpd-2)+bdt2*pd(jpd-1)+dd*pd(jpd);
                end 
                
                %Calculamos los coeficientes del polinomio.
                
                p00 = z(1);
                p10 = zu(1);
                p01 = zv(1);
                p20 = 0.5D+00*zuu(1);
                p11 = zuv(1);
                p02 = 0.5D+00*zvv(1);
                h1 = z(2)-p00-p01-p02;
                h2 = zv(2)-p01-zvv(1);
                h3 = zvv(2)-zvv(1);
                p03 =  10*h1-4*h2+0.5*h3;
                p04 = -15*h1+7*h2-h3;
                p05 =   6*h1-3*h2+0.5*h3;
                h1 = zu(2)-p10-p11;
                h2 = zuv(2)-p11;
                p12 =  3*h1-h2;
                p13 = -2*h1+h2;
                p21 = 0;
                p23 = -zuu(2)+zuu(1);
                p22 = -1.5*p23;
                itpv = it0;
                
                %Covertimos xii y yii a sistema UV.
                
                dx = xii-x0;
                dy = yii-y0;
                u = ap*dx+bp*dy;
                v = cp*dx+dp*dy;
                
                %Evaluamos el polinomio.
                
                p0 = p00+v*(p01+v*(p02+v*(p03+v*(p04+v*p05))));
                p1 = p10+v*(p11+v*(p12+v*p13));
                p2 = p20+v*(p21+v*(p22+v*p23));
				
				%Encontramos el punto buscado.
				
                zii = p0+u*(p1+u*p2);
                return
            elseif(it0==itpv)
                
                %Covertimos xii y yii a sistema UV.
                
                dx = xii-x0;
                dy = yii-y0;
                u = ap*dx+bp*dy;
                v = cp*dx+dp*dy;
                
                %Evaluamos el polinomio.
                
                p0 = p00+v*(p01+v*(p02+v*(p03+v*(p04+v*p05))));
                p1 = p10+v*(p11+v*(p12+v*p13));
                p2 = p20+v*(p21+v*(p22+v*p23));
				
				%Encontramos el punto buscado.
				
                zii = p0+u*(p1+u*p2);
                return
            end
        else
            if ( it0 ~= itpv ) 
                
                %Usamos las derivadas parciales.
                
                jipl = 3*il2-2;
                idp = ipl(jipl);
                x(1) = xd(idp);
                y(1) = yd(idp);
                z(1) = zd(idp);
                jpdd = 5*(idp-1);
                
                for kpd = (1:5)
                    jpdd = jpdd+1;
                    pd(kpd) = pdd(jpdd);
                end 
                
                %Calculamos los coeficientes del polinomio.
               
                p00 = z(1);
                p10 = pd(1);
                p01 = pd(2);
                p20 = 0.5*pd(3);
                p11 = pd(4);
                p02 = 0.5*pd(5);
                itpv = it0;
                
                %Convertimos xii y yii al sistema UV.
                u = xii-x(1);
                v = yii-y(1);
                
                %Evaluamos el polinomio. 
                
                 p0 = p00+v*(p01+v*p02);
                 p1 = p10+v*p11;
                 zii = p0+u*(p1+u*p20);
            else
                
                %Convertimos xii y yii al sistema UV.
                u = xii-x(1);
                v = yii-y(1);
                
                %Evaluamos el polinomio.
                
                 p0 = p00+v*(p01+v*p02);
                 p1 = p10+v*p11;
				 
				 %Encontramos el punto buscado.
                 
				 zii = p0+u*(p1+u*p20);
            
            end
			
        end
    end
end

%Parámetros de Salida:

%zii:Valor de la Interpolación Bivariada de Akima en el punto (xii,yii).

