%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                          %             
%   Función que crea una malla triangular  %
%                                          %                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%Parámetros de Entrada:

%xd,yd:Vectores que guardan los números de (xi,yi).

function[nt, ipt, nl, ipl, iwl, iwp, wk]=idtang(xd,yd)
    
    %Funciones para cálculos de la malla.
    
    function[d]=dsqf(u1,v1,u2,v2)
        d=(u2-u1)^2+(v2-v1)^2;
    end
    function[lado]=side(u1,v1,u2,v2,u3,v3)
        lado=(v3-v1)*(u2-u1)-(u3-u1)*(v2-v1);
    end
    
    %Proceso preeliminar.
    
	ratio=10^(-6);
	nrep = 100;
    itf=zeros(2,1);
	ndp=length(xd);
	ipt=zeros(6*ndp-15,1);
	ipl=zeros(6*ndp,1);
	iwl=zeros(18*ndp,1);
	iwp=zeros(ndp,1);
	wk=zeros(ndp,1);
	ndp0=ndp;
	ndpm1=ndp0-1;
    
	%Determinamos el par de puntos más cercanos y el punto medio.
	
	dsqmn = dsqf(xd(1),yd(1),xd(2),yd(2));
	ipmn1 = 1;
    ipmn2 = 2;
	for ip1=(1:ndpm1)
		x1 = xd(ip1);
        y1 = yd(ip1);
        ip1p1 = ip1+1;
		for ip2=(ip1p1:ndp0)
			dsqi = dsqf(x1,y1,xd(ip2),yd(ip2));
            if ( dsqi < dsqmn )
				dsqmn = dsqi;
				ipmn1 = ip1;
				ipmn2 = ip2;
            end
		end
	end
	dsq12 = dsqmn;
    xdmp = (xd(ipmn1)+xd(ipmn2))/2;
    ydmp = (yd(ipmn1)+yd(ipmn2))/2;
    
	%Ordenamos los ndp-2 puntos restantes de forma ascendente con
	%respecto a la distancia del punto medio 
	%calculado anteriormente.
	
	jp1 = 2;
    for ip1 = (1:ndp0)
        if ( ip1 ~= ipmn1 && ip1 ~= ipmn2 )
			jp1 = jp1+1;
			iwp(jp1) = ip1;
			wk(jp1) = dsqf(xdmp,ydmp,xd(ip1),yd(ip1));
        end
    end
    for jp1 = (3:ndpm1)
        dsqmn = wk(jp1);
        jpmn = jp1;
        for jp2 = (jp1:ndp0)
			if ( wk(jp2) < dsqmn )
				dsqmn = wk(jp2);
				jpmn = jp2;
			end 
        end
        its = iwp(jp1);
        iwp(jp1) = iwp(jpmn);
        iwp(jpmn) = its;
        wk(jpmn) = wk(jp1);
    end
    
    %Modificamos los puntos de tal manera que los primeros tres puntos
	%no sean colineales.
	
	ar = dsq12*ratio;
    x1 = xd(ipmn1);
    y1 = yd(ipmn1);
    dx21 = xd(ipmn2)-x1;
    dy21 = yd(ipmn2)-y1;
	for jp=(3:ndp0)
		ip = iwp(jp);
		if(abs((yd(ip)-y1)*dx21-(xd(ip)-x1)*dy21)>ar) 
			break;
		end	
	end
	
    if ( jp ~= 3 ) 
        jpmx = jp;
        jp = jpmx+1;
        for jpc =(4:jpmx)
			jp = jp-1;
			iwp(jp) = iwp(jp-1);
        end 
        iwp(3) = ip;
    end
    
    %Para el primer triangulo guardamos
	%en número de vértices en ipt y los
	%bordes en ipl.
	
    ip1 = ipmn1;
    ip2 = ipmn2;
    ip3 = iwp(3);
    if(side(xd(ip1),yd(ip1),xd(ip2),yd(ip2),xd(ip3),yd(ip3))<0)
		ip1 = ipmn2;
		ip2 = ipmn1;
    end

	nt0 = 1;
    ntt3 = 3;
    ipt(1) = ip1;
    ipt(2) = ip2;
    ipt(3) = ip3;
    nl0 = 3;
    nlt3 = 9;
    ipl(1) = floor(ip1);
    ipl(2) = floor(ip2);
    ipl(3) = floor(1);
    ipl(4) = floor(ip2);
    ipl(5) = floor(ip3);
    ipl(6) = floor(1);
    ipl(7) = floor(ip3);
    ipl(8) = floor(ip1);
    ipl(9) = 1;
	
    %Seguimos el proceso con los ndp-3
	%puntos restantes.
	
	for jp1=(4:ndp0)
		ip1 = iwp(jp1);
        x1 = xd(ip1);
        y1 = yd(ip1);
        
        %Determinamos los bordes de los
        %segmentos de linea.
		
        ip2 = ipl(1);
        jpmn = 1;
        dxmn = xd(ip2)-x1;
        dymn = yd(ip2)-y1;
        dsqmn = dxmn^2+dymn^2;
        armn = dsqmn*ratio;
        jpmx = 1;
        dxmx = dxmn;
        dymx = dymn;
        dsqmx = dsqmn;
        armx = armn;
        for jp2=(2:nl0)
		    ip2 = ipl(3*jp2-2);
            dx = xd(ip2)-x1;
            dy = yd(ip2)-y1;
            ar = dy*dxmn-dx*dymn;
           if(ar<=armn)
				dsqi = dx^2+dy^2;
                if(~(ar>=(-armn)&& dsqi>=dsqmn)) 
					jpmn = jp2;
					dxmn = dx;
					dymn = dy;
					dsqmn = dsqi;
					armn = dsqmn*ratio;
                end
            end
		
			ar = dy*dxmx-dx*dymx;
            if(ar>=(-armx))
				dsqi = dx^2+dy^2;
                if(~(ar<=armx && dsqi>=dsqmx))
					jpmx = jp2;
					dxmx = dx;
					dymx = dy;
					dsqmx = dsqi;
					armx = dsqmx*ratio;
                end
             end			
		
        end
        if(jpmx<jpmn)
			jpmx = jpmx+nl0;
        end
		nsh = jpmn-1;
		
        %Seguimos obteniento los nuevos bordes para guardar en
		%ipl.
		
        if(nsh>0)
			nsht3 = nsh*3;
			for jp2t3 = (3:3:nsht3)
				jp3t3 = jp2t3+nlt3;
				ipl(jp3t3-2) = ipl(jp2t3-2);
				ipl(jp3t3-1) = ipl(jp2t3-1);
				ipl(jp3t3)   = ipl(jp2t3);
			end
			for jp2t3 = (3:3:nlt3)
				jp3t3 = jp2t3+nsht3;
				ipl(jp2t3-2) = ipl(jp3t3-2);
				ipl(jp2t3-1) = ipl(jp3t3-1);
				ipl(jp2t3)   = ipl(jp3t3);
			end 
			jpmx = jpmx-nsh;
        end
        
        %Seguimos creando más triangulos
		%y actualizamos los bordes de 
		%cada triangulo usando ipt,ipl,iwl.
		
		jwl=0;
		for jp2 = (jpmx:nl0)
		    jp2t3 = jp2*3;
            ipl1 = ipl(jp2t3-2);
            ipl2 = ipl(jp2t3-1);
            it = ipl(jp2t3);
            
            %Tomamos los triangulos de ipt.
		    
            nt0 = nt0+1;
			ntt3 = ntt3+3;
			ipt(ntt3-2) = ipl2;
			ipt(ntt3-1) = ipl1;
			ipt(ntt3)   = ip1;
            
            %Actualizamos los bordes.
			
            if(jp2==jpmx)
			    ipl(jp2t3-1) = ip1;
				ipl(jp2t3)= nt0;
            end
			%61
            if(jp2==nl0)
			    nln = jpmx+1;
				nlnt3 = nln*3;
				ipl(nlnt3-2) = ip1;
				ipl(nlnt3-1) = ipl(1);
				ipl(nlnt3)   = nt0;
            end
            
            %Determinamos los vértices que
            %no están sobre el borde.
			
            itt3 = it*3;
			ipti = ipt(itt3-2);
            if(~(ipti~=ipl1 && ipti~=ipl2))
				ipti = ipt(itt3-1);
				if(~(ipti~=ipl1&&ipti~=ipl2))
					ipti = ipt(itt3);
				end
            end
            
            %Revisamos el intercambio si es necesario.
			
            if(idxchg(xd,yd,ip1,ipti,ipl1,ipl2)~=0)
			
                %Modificamos ipt cuando sea necesario.
				ipt(itt3-2) = ipti;
				ipt(itt3-1) = ipl1;
				ipt(itt3)   = ip1;
				ipt(ntt3-1) = ipti;
                if(jp2==jpmx)
					ipl(jp2t3) = it;
                end
                if(jp2==nl0&&ipl(3)==it)
					ipl(3) = nt0;
                end
                
                %Usamos iwl.
				
                jwl = jwl+4;
				iwl(jwl-3) = ipl1;
				iwl(jwl-2) = ipti;
				iwl(jwl-1) = ipti;
				iwl(jwl)   = ipl2;
            end	

		end
		nl0 = nln;
        nlt3 = nlnt3;
        nlf = jwl/2;
        
		%Continua la triangulación.
		
        if(nlf~=0)

			ntt3p3 = ntt3+3;
            for irep = (1:nrep)
				for ilf = (1:nlf)
					ilft2 = ilf*2;
					ipl1 = iwl(ilft2-1);
					ipl2 = iwl(ilft2);
                    
                    %Buscamos en ipt dos lados que que coincidan.
					
                    ntf=0;
                    for itt3r =(3:3:ntt3)
						itt3 = ntt3p3-itt3r;
						ipt1 = ipt(itt3-2);
						ipt2 = ipt(itt3-1);
						ipt3 = ipt(itt3);
						if(~(ipl1~=ipt1&&ipl1~=ipt2&&ipl1~=ipt3))
							 if(~(ipl2~=ipt1&&ipl2~=ipt2 &&ipl2~=ipt3))
								ntf = ntf+1;
								itf(ntf) = itt3/3;
								if(ntf==2)
									break;
								end
							 end
						end		

                    end
                    
                    %Determinamos los vértices del triangulo.
					
                    if(ntf>=2)
					
						it1t3 = itf(1)*3;
						ipti1 = ipt(it1t3-2);
						if(~(ipti1~=ipl1&&ipti1~=ipl2))
							ipti1 = ipt(it1t3-1);
							if(~(ipti1~=ipl1&&ipti1~=ipl2))
								ipti1 = ipt(it1t3);
							end
						end
					
						it2t3 = itf(2)*3;
						ipti2 = ipt(it2t3-2);
                        if(~(ipti2~=ipl1&&ipti2~=ipl2))
							ipti2 = ipt(it2t3-1);
							if(~(ipti2~=ipl1&&ipti2~=ipl2))
								ipti2 = ipt(it2t3);
							end
                        end
                        
                        %Revisamos nuevamente.
						
                         if(~(idxchg(xd,yd,ipti1,ipti2,ipl1,ipl2)==0))
						
                            %Modificamos ipt si es necesario. 
						    
                            ipt(it1t3-2) = ipti1;
							ipt(it1t3-1) = ipti2;
							ipt(it1t3)   = ipl1;
							ipt(it2t3-2) = ipti2;
							ipt(it2t3-1) = ipti1;
							ipt(it2t3)   = ipl2;
                            
                            %Creamos nuevos marcadores.
							
                            jwl = jwl+8;
							iwl(jwl-7) = ipl1;
							iwl(jwl-6) = ipti1;
							iwl(jwl-5) = ipti1;
							iwl(jwl-4) = ipl2;
							iwl(jwl-3) = ipl2;
							iwl(jwl-2) = ipti2;
							iwl(jwl-1) = ipti2;
							iwl(jwl)   = ipl1;
                            for jlt3 = (3:3:nlt3)
								iplj1 = ipl(jlt3-2);
								iplj2 = ipl(jlt3-1);
                                if((iplj1==ipl1&&iplj2==ipti2)||(iplj2==ipl1&&iplj1==ipti2))
									ipl(jlt3) = itf(1);
                                end
                                if((iplj1==ipl2&&iplj2==ipti1)||(iplj2==ipl2&&iplj1==ipti1))
									ipl(jlt3) = itf(2);
                                 end
								
                            end
							
                         end 
                    end
					
				
				end
				nlfc = nlf;
				nlf = jwl/2;
				if(nlf~=nlfc)
				
                    %Reiniciamos iwl.
					
                    jwl = 0;
					jwl1mn = (nlfc+1)*2;
					nlft2 = nlf*2;
					for jwl1 =( jwl1mn:2:nlft2)
                        jwl = jwl+2;
						iwl(jwl-1) = iwl(jwl1-1);
						iwl(jwl)   = iwl(jwl1);
					end
					nlf = jwl/2;
				else
					break;
				end
			
            end
			
        end
		
		
		

	end

	for itt3 = (3:3:ntt3)
		ip1 = ipt(itt3-2);
        ip2 = ipt(itt3-1);
        ip3 = ipt(itt3);
		if(side(xd(ip1),yd(ip1),xd(ip2),yd(ip2),xd(ip3),yd(ip3))<0)
			ipt(itt3-2) = ip2;
            ipt(itt3-1) = ip1;
		end
	end
	nt = nt0;
    nl = nl0;
end

%Parámetros de Salida:

%nt:Número de triángulos.

%ipt: Índice de los vértices de los triángulos.

%nl: Número de lineas que conforman en borde de la triangular.

%ipl: Índice de los puntos que conforman las nl

%iwl,iwp,iwk: Vectores que guardan información de la malla.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                              %             
%   Función para el intercambio de triángulos  %
%                                              %                      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Parámetros de Entrada:

%x,y:Coordenadas (xi,yi).

%i1,i2,i3,i4: Índices de los puntos del cuadrilatero y la diagonal.

function[idx]=idxchg( x, y, i1, i2, i3, i4 )

  %Proceso preeliminar.
  
  x1 = x(i1);
  y1 = y(i1);
  x2 = x(i2);
  y2 = y(i2);
  x3 = x(i3);
  y3 = y(i3);
  x4 = x(i4);
  y4 = y(i4);
  
  %Cálculos.
  
  idx = 0;
  u3 = ( y2 - y3 ) * ( x1 - x3 ) - ( x2 - x3 ) * ( y1 - y3 );
  u4 = ( y1 - y4 ) * ( x2 - x4 ) - ( x1 - x4 ) * ( y2 - y4 );
  if ( 0 < u3 * u4 )

    u1 = (y3-y1)*(x4-x1)-(x3-x1)*(y4-y1);
    u2 = (y4-y2)*(x3-x2)-(x4-x2)*(y3-y2);

    a1sq = (x1-x3)^2+(y1-y3)^2;
    b3sq = a1sq;
    b1sq = (x4-x1)^2+(y4-y1)^2;
    a4sq = b1sq;
    c1sq = (x3-x4)^2+(y3-y4)^2;
    c2sq = c1sq;
    a2sq = (x2-x4)^2+(y2-y4)^2;
    b4sq = a2sq;
    b2sq = (x3-x2)^2+(y3-y2)^2;
    a3sq = b2sq;
    c3sq = (x2-x1)^2+(y2-y1)^2;
    c4sq = c3sq;

    s1sq = u1 * u1 / ( c1sq * max ( a1sq, b1sq ) );
    s2sq = u2 * u2 / ( c2sq * max ( a2sq, b2sq ) );
    s3sq = u3 * u3 / ( c3sq * max ( a3sq, b3sq ) );
    s4sq = u4 * u4 / ( c4sq * max ( a4sq, b4sq ) );

    if ( min ( s1sq, s2sq ) < min ( s3sq, s4sq ) ) 
      idx = 1;
    end 

  end 
  
end

%Parámetros de Salida:

%idx: Indicador del intercambio de triángulos. 
